definition module Corba

import StdEnv, StdDebug

from StdString import String;

////////////////////////////////
//       UTILITIES
////////////////////////////////

:: ResultOrException a b = Result a | Exception b

// If there was an exception, the result is CorbaVoid
// Luckily, lazy evaluation prevents the system from trying to decode
// it into another CORBA type...
corba_result_create :: .a CorbaException -> ResultOrException .a CorbaException

:: MayBe a = Just a | Nothing

just :: (MayBe u:a) -> u:a

is_just :: (MayBe .a) -> Bool

toStr :: !.a -> String

map_with_env :: (a *env -> (b, *env)) [a] *env -> ([b], *env)

map_maybe_with_env :: (a *env -> (MayBe b, *env)) [a] *env -> (MayBe [b], *env)

////////////////////////////////
//        TYPECODE
////////////////////////////////

:: TypeCode = 	Tk_Null | 
				Tk_Void | 
				Tk_Short | 
				Tk_Long | 
				Tk_UShort |
				Tk_ULong | 
				Tk_Float | 
				Tk_Double | 
				Tk_Boolean | 
				Tk_Char |
				Tk_Octet | 
				Tk_Any | 
				Tk_String Int | 
				Tk_TypeCode | 
				Tk_ObjRef String String | 
				Tk_Struct String String [(String, TypeCode)] | 
			  	Tk_Sequence Int TypeCode |
			  	Tk_Union String String TypeCode [(String, Any, TypeCode)] |
				Tk_Enum String String [String] |
				Tk_Alias String String TypeCode |
				Tk_Array Int TypeCode |
				Tk_Exception String String [(String, TypeCode)] |
				Tk_Recursive String

instance == TypeCode

////////////////////////////////
//          CORBA::Any
////////////////////////////////

:: Any = 	CorbaNull | 
			CorbaVoid | 
			CorbaShort Int | 
			CorbaLong Int |
		   	CorbaUShort Int | 
			CorbaULong Int | 
			CorbaFloat Real |
			CorbaDouble Real |
			CorbaBoolean Bool |
			CorbaChar Char |
			CorbaOctet Int |
			CorbaString String |
			CorbaAny Any |
			CorbaTypeCode TypeCode |
			CorbaObjRef CORBA_Object |
			CorbaStruct TypeCode [(String, Any)] |  // tc [(name, type)]
			CorbaSequence TypeCode [Any] |			// tc [members]
			CorbaUnion TypeCode Any Any	|			// tc discriminator value
			CorbaEnum TypeCode String |				// tc value
			CorbaArray TypeCode [Any] |				// tc [members]
			CorbaException TypeCode [(String, Any)]
//
// If there is no active element in a CorbaUnion, the second argument
// is CorbaNull.
//

////////////////////////////////
//        EXCEPTION
////////////////////////////////

:: CompletionStatus = Completed_Yes | Completed_No | Completed_Maybe
:: CorbaException = NoException | 
					SystemException String CompletionStatus Int | 
					UserException String Any

corba_exception_encode :: CorbaException -> Any

////////////////////////////////
//         NAMEDVALUE
////////////////////////////////

// The xxxTemplate constructors are used to give the name and type of
// arguments, without specifying a value
:: CorbaArg = 	InArg String Any 
			| 	OutArg String Any 
			| 	InOutArg String Any
			| 	InArgTemplate String TypeCode
			| 	OutArgTemplate String TypeCode
			| 	InOutArgTemplate String TypeCode

corba_arg_get_value :: CorbaArg -> Any

////////////////////////////////
//          REQUEST
////////////////////////////////

corba_invoke :: CORBA_Object String [CorbaArg] TypeCode [TypeCode] *World -> (Any, CorbaException, [CorbaArg], *World)

////////////////////////////////
//        CORBA::Void
////////////////////////////////

// There is no void type in Clean
:: CORBA_Void	:==	Void

CORBA_Void__tc	::	TypeCode

CORBA_Void__decode :: Any *World	->	(MayBe CORBA_Void, *World)

CORBA_Void__encode :: CORBA_Void -> Any

////////////////////////////////
//        CORBA::Short
////////////////////////////////

:: CORBA_Short	:==	Int

CORBA_Short__tc	::	TypeCode

CORBA_Short__decode :: Any *World	->	(MayBe CORBA_Short, *World)

CORBA_Short__encode :: CORBA_Short -> Any

////////////////////////////////
//        CORBA::Long
////////////////////////////////

:: CORBA_Long	:==	Int

CORBA_Long__tc	::	TypeCode

CORBA_Long__decode :: Any *World	->	(MayBe CORBA_Long, *World)

CORBA_Long__encode :: CORBA_Long -> Any

////////////////////////////////
//        CORBA::UShort
////////////////////////////////

:: CORBA_UShort	:==	Int

CORBA_UShort__tc	::	TypeCode

CORBA_UShort__decode :: Any *World	->	(MayBe CORBA_UShort, *World)

CORBA_UShort__encode :: CORBA_UShort -> Any

////////////////////////////////
//        CORBA::ULong
////////////////////////////////

:: CORBA_ULong	:==	Int

CORBA_ULong__tc	::	TypeCode

CORBA_ULong__decode :: Any *World	->	(MayBe CORBA_ULong, *World)

CORBA_ULong__encode :: CORBA_ULong -> Any

////////////////////////////////
//        CORBA::Float
////////////////////////////////

:: CORBA_Float	:==	Real

CORBA_Float__tc	::	TypeCode

CORBA_Float__decode :: Any *World	->	(MayBe CORBA_Float, *World)

CORBA_Float__encode :: CORBA_Float -> Any

////////////////////////////////
//        CORBA::Double
////////////////////////////////

:: CORBA_Double	:==	Real

CORBA_Double__tc	::	TypeCode

CORBA_Double__decode :: Any *World	->	(MayBe CORBA_Double, *World)

CORBA_Double__encode :: CORBA_Double -> Any

////////////////////////////////
//        CORBA::Boolean
////////////////////////////////

:: CORBA_Boolean	:==	Bool

CORBA_Boolean__tc	::	TypeCode

CORBA_Boolean__decode :: Any *World	->	(MayBe CORBA_Boolean, *World)

CORBA_Boolean__encode :: CORBA_Boolean -> Any

////////////////////////////////
//        CORBA::Char
////////////////////////////////

:: CORBA_Char	:==	Char

CORBA_Char__tc	::	TypeCode

CORBA_Char__decode :: Any *World	->	(MayBe CORBA_Char, *World)

CORBA_Char__encode :: CORBA_Char -> Any

////////////////////////////////
//        CORBA::Octet
////////////////////////////////

:: CORBA_Octet	:==	Int

CORBA_Octet__tc	::	TypeCode

CORBA_Octet__decode :: Any *World	->	(MayBe CORBA_Octet, *World)

CORBA_Octet__encode :: CORBA_Octet -> Any

////////////////////////////////
//        CORBA::Any
////////////////////////////////

:: CORBA_Any	:==	Any

CORBA_Any__tc	::	TypeCode

CORBA_Any__decode :: Any *World	->	(MayBe CORBA_Any, *World)

CORBA_Any__encode :: Any -> Any

////////////////////////////////
//     CORBA::TypeCode
////////////////////////////////

:: CORBA_TypeCode	:==	TypeCode

CORBA_TypeCode__tc :: TypeCode

CORBA_TypeCode__decode :: Any *World -> (MayBe CORBA_TypeCode, *World)

CORBA_TypeCode__encode :: TypeCode -> Any

////////////////////////////////
//      CORBA::String
////////////////////////////////

// This is the unbounded version
:: CORBA_String	:==	String

CORBA_String__tc :: TypeCode

CORBA_String__decode :: Any *World -> (MayBe CORBA_String, *World)

CORBA_String__encode :: CORBA_String -> Any

////////////////////////////////
//      CORBA::Object
////////////////////////////////

//
// The object references are not unique, because if they were, EVERYTHING
// which contains object references (Any etc.) would have to be unique.
// Instead, the unique *World argument passed to 'corba_invoke' is used to
// serialize object invocations.
// Problem: Since the reference is not unique, it can be used after it has
// been destroyed.
// Solution: ?
//

:: CORBA_Object

CORBA_Object__create :: CORBA_Object -> CORBA_Object

CORBA_Object__tc :: TypeCode

CORBA_Object__decode :: Any *World -> (MayBe CORBA_Object, *World)

CORBA_Object__encode :: CORBA_Object -> Any

CORBA_Object_is_a :: CORBA_Object String *World -> (Bool, *World)

CORBA_Object__nil :: *World -> (CORBA_Object, *World)

class CORBA_Object__widen_class a where
	CORBA_Object__widen :: a -> CORBA_Object

instance CORBA_Object__widen_class CORBA_Object

////////////////////////////////
//         ORB
////////////////////////////////

:: CORBA_ORB

CORBA_ORB_init :: [String] *World -> (CORBA_ORB, [String], *World)

CORBA_ORB_bind :: CORBA_ORB String String *World -> (MayBe CORBA_Object, *World)

CORBA_ORB_string_to_object :: CORBA_ORB String *World -> (MayBe CORBA_Object, *World)

CORBA_ORB_object_to_string :: CORBA_ORB CORBA_Object *World -> (String, *World)

CORBA_ORB_resolve_initial_references :: CORBA_ORB String *World -> (MayBe CORBA_Object, *World)

// Write the stringified IOR of the object to the specified file
WriteIORToFile :: CORBA_ORB CORBA_Object String *World -> *World

// Read a stringified IOR from the given file
ReadIORFromFile :: CORBA_ORB String *World -> (MayBe CORBA_Object, *World)

////////////////////////////////
//      SERVER REQUEST
////////////////////////////////

:: CORBA_ServerRequest

CORBA_ServerRequest_operation :: CORBA_ServerRequest *env -> (String, *env)

CORBA_ServerRequest_set_arg_types :: CORBA_ServerRequest [CorbaArg] *World -> *World

CORBA_ServerRequest_get_arg :: CORBA_ServerRequest Int *World -> (Any, *World) 

CORBA_ServerRequest_set_arg :: CORBA_ServerRequest Int Any *World -> *World

CORBA_ServerRequest_set_result :: CORBA_ServerRequest Any *World -> *World

CORBA_ServerRequest_set_exception :: CORBA_ServerRequest Any *World -> *World

////////////////////////////////
//    SERVER SIDE BINDING
////////////////////////////////

//
// The server side binding uses a simplified version of the ObjectIO
// library.
//

// The process state
:: PSt l
	= {	ls :: l
	,	io :: IOSt l
	}

// The private IO state
:: IOSt l

// Initialization function called by CORBA_Server_run
:: ServerInitFunc l *env :== (PSt l) env -> ((PSt l), env)

// The empty initialization function			
NoInit :: (PSt l) *env -> ((PSt l), *env)

// Empty local state
:: Void = Void

// Wrapper function around the servants created by the programmer
// Can be used to support DSI (Dynamic Skeleton Interface)
:: CORBA_Servant_Wrapper l *env = E.obj: {
	// The servant supplied by the programmer
	obj 		:: obj,
	// Function to dispatch requests to the servant supplied by the
	// programmer
	dispatch 	:: (PSt l) CORBA_ServerRequest obj env -> (((PSt l), obj), env)
	};

// This function runs the CORBA server process
// It does the following:
// - It initializes the local process state from the first argument
// - It evaluates the second argument, producing an initial process state
// - It server CORBA requests until the CORBA_Server_exit function is
//   evaluated
CORBA_Server_run :: CORBA_ORB l (ServerInitFunc l *World) *World -> *World

// Instruct the CORBA server process to exit
CORBA_Server_exit :: (PSt l) -> (PSt l)

// Registers a new CORBA servant
// CORBA_Server_add_servant ps repoid servant -> ps
CORBA_Server_add_servant :: (PSt l) String (CORBA_Servant_Wrapper l *World) *World -> (CORBA_Object, (PSt l), *World)

// Returns the reference of the currently executing object
// Only valid during the serving of a request
CORBA_Server_get_current :: (PSt l) -> CORBA_Object

// Returns the ORB object
CORBA_Server_get_orb :: (PSt l) -> CORBA_ORB

// Local Variables:
// tab-width: 4
// End:

