implementation module corba_wrappers

import StdEnv, StdDebug

from StdString import String;

//
// NOTE: We use the 'A' type code to specify the type of the *env parameter.
// This is used by Object IO 1.2, and it seems to work.
//

////////////////////////////////
//        CStringArray
////////////////////////////////

:: CStringArrayPtr :== Int;

c_string_array_create :: !Int !*env -> (!CStringArrayPtr, !*env);
c_string_array_create size env = code {
	ccall c_string_array_create "I:VI:A"
	}

c_string_array_set :: !CStringArrayPtr !Int !String !*env -> !*env;
c_string_array_set arr index value env = code {
	ccall c_string_array_set "IIS:V:A"
	}

c_string_array_get :: !CStringArrayPtr !Int !*env -> (!String, !*env);
c_string_array_get arr index env = code {
	ccall c_string_array_get "II:VS:A"
	}

c_string_array_delete :: !CStringArrayPtr !*env -> !*env
c_string_array_delete arr env = code {
	ccall c_string_array_delete "I:V:A"
	}

////////////////////////////////
//         ORB
////////////////////////////////

:: ORBPtr :== Int;

corba_orb_init :: (!CStringArrayPtr, !Int) !*env -> (!ORBPtr, !CStringArrayPtr, !Int, !*env)
corba_orb_init (argv, argc) env = code {
	ccall corba_orb_init "II:VIII:A"
	}

:: CorbaObjectPtr :== Int

corba_orb_string_to_object :: !ORBPtr !String !*env -> (!CorbaObjectPtr, !*env)
corba_orb_string_to_object orb string env = code {
	ccall corba_orb_string_to_object "IS:VI:A"
	}

corba_orb_bind :: !ORBPtr !String !String !*env -> (!CorbaObjectPtr, !*env)
corba_orb_bind orb repoid address env = code {
	ccall corba_orb_bind "ISS:VI:A"
	}

corba_orb_object_to_string :: !ORBPtr !CorbaObjectPtr !*env -> (!String, !*env)
corba_orb_object_to_string orb obj env = code {
	ccall corba_orb_object_to_string "II:VS:A"
	}

corba_orb_resolve_initial_references :: !ORBPtr !String !*env -> (!CorbaObjectPtr, !*env)
corba_orb_resolve_initial_references orb name env = code {
	ccall corba_orb_resolve_initial_references "IS:VI:A"
	}

corba_object_is_a :: !CorbaObjectPtr !String !*env -> (!Bool, !*env)
corba_object_is_a obj repoid env = code {
	ccall corba_object_is_a "IS:VI:A"
	}

corba_object_nil :: !*env -> (!CorbaObjectPtr, !*env)
corba_object_nil env = code {
	ccall corba_object_nil ":VI:A"
	}

corba_object_hash :: !CorbaObjectPtr !*env -> (!Int, !*env)
corba_object_hash obj env = code {
	ccall corba_object_hash "I:VI:A"
	}

/////////////////////////////////
//        TYPECODE
////////////////////////////////

// These should be equal to the TCKind values used by the ORB 
// implementation
TK_NULL 	:== 0
TK_VOID 	:== 1
TK_SHORT 	:== 2
TK_LONG 	:== 3
TK_USHORT 	:== 4
TK_ULONG 	:== 5
TK_FLOAT 	:== 6
TK_DOUBLE 	:== 7
TK_BOOLEAN 	:== 8
TK_CHAR 	:== 9
TK_OCTET 	:== 10
TK_ANY 		:== 11
TK_TYPECODE :== 12
TK_PRINCIPAL:== 13
TK_OBJREF	:== 14
TK_STRUCT	:== 15
TK_UNION	:== 16
TK_ENUM		:== 17
TK_STRING	:== 18
TK_SEQUENCE :== 19
TK_ARRAY	:== 20
TK_ALIAS	:== 21
TK_EXCEPT	:== 22

// The other typecodes (TK_FIXED etc., !*env) are not supported

:: TypeCodePtr :== Int

// Aux structure used to construct IDL structures

:: StructMemberSeqPtr :== Int

corba_struct_member_seq_create :: !Int !*env -> (!StructMemberSeqPtr, !*env)
corba_struct_member_seq_create size env = code {
	ccall corba_struct_member_seq_create "I:VI:A"
	}

// Fill a member of the sequence
corba_struct_member_seq_fill :: !StructMemberSeqPtr !Int !String !TypeCodePtr !*env -> !*env
corba_struct_member_seq_fill seq index name type env = code {
	ccall corba_struct_member_seq_fill "IISI:V:A"
	}

corba_struct_member_seq_delete :: !StructMemberSeqPtr !*env -> !*env
corba_struct_member_seq_delete seq env = code {
	ccall corba_struct_member_seq_delete "I:V:A"
	}

// Aux structure used to construct IDL unions

:: UnionMemberSeqPtr :== Int

corba_union_member_seq_create :: !Int !*env -> (!UnionMemberSeqPtr, !*env)
corba_union_member_seq_create size env = code {
	ccall corba_union_member_seq_create "I:VI:A"
	}

// Fill a member of the sequence
corba_union_member_seq_fill :: !UnionMemberSeqPtr !Int !String !AnyPtr !TypeCodePtr !*env -> !*env
corba_union_member_seq_fill seq index name label type env = code {
	ccall corba_union_member_seq_fill "IISII:V:A"
	}

corba_union_member_seq_delete :: !UnionMemberSeqPtr !*env -> !*env
corba_union_member_seq_delete seq env = code {
	ccall corba_union_member_seq_delete "I:V:A"
	}

// Aux structure used to construct IDL enums

:: EnumMemberSeqPtr :== Int

corba_enum_member_seq_create :: !Int !*env -> (!EnumMemberSeqPtr, !*env)
corba_enum_member_seq_create size env = code {
	ccall corba_enum_member_seq_create "I:VI:A"
	}

corba_enum_member_seq_fill :: !EnumMemberSeqPtr !Int !String !*env -> !*env
corba_enum_member_seq_fill seq index name env = code {
	ccall corba_enum_member_seq_fill "IIS:V:A"
	}

corba_enum_member_seq_delete :: !EnumMemberSeqPtr !*env -> !*env
corba_enum_member_seq_delete seq env = code {
	ccall corba_enum_member_seq_delete "I:V:A"
	}

// Create a TypeCode which does not require additional parameters
corba_tc_create_basic :: !Int !*env -> (!TypeCodePtr, !*env)
corba_tc_create_basic kind env = code {
	ccall corba_tc_basic "I:VI:A"
	}

// Create a TypeCode of type TK_STRING
corba_tc_create_string :: !Int !*env -> (!TypeCodePtr, !*env)
corba_tc_create_string bound env = code {
	ccall corba_tc_string "I:VI:A"
	}

// Create a TypeCode of type TK_OBJREF
corba_tc_create_objref :: !String !String !*env -> (!TypeCodePtr, !*env)
corba_tc_create_objref repoid name env = code {
	ccall corba_tc_objref "SS:VI:A"
	}

// Create a TypeCode of type TK_STRUCT
corba_tc_create_struct :: !String !String !StructMemberSeqPtr !*env -> (!TypeCodePtr, !*env)
corba_tc_create_struct repoid name members env = code {
	ccall corba_tc_struct "SSI:VI:A"
	}

// Create a TypeCode of type TK_SEQUENCE
corba_tc_create_sequence :: !Int !TypeCodePtr !*env -> (!TypeCodePtr, !*env)
corba_tc_create_sequence bound content_type env = code {
	ccall corba_tc_sequence "II:VI:A"
	}

// Create a TypeCode of type TK_UNION
corba_tc_create_union :: !String !String !TypeCodePtr !UnionMemberSeqPtr !*env -> (!TypeCodePtr, !*env)
corba_tc_create_union repoid name disc_type members env = code {
	ccall corba_tc_union "SSII:VI:A"
	}

// Create a TypeCode of type TK_ENUM
corba_tc_create_enum :: !String !String !EnumMemberSeqPtr !*env -> (!TypeCodePtr, !*env)
corba_tc_create_enum repoid name members env = code {
	ccall corba_tc_enum "SSI:VI:A"
	}

corba_tc_create_alias :: !String !String !TypeCodePtr !*env -> (!TypeCodePtr, !*env)
corba_tc_create_alias repoid name members env = code {
	ccall corba_tc_alias "SSI:VI:A"
	}

corba_tc_create_array :: !TypeCodePtr !Int !*env -> (!TypeCodePtr, !*env)
corba_tc_create_array content_type length env = code {
	ccall corba_tc_array "II:VI:A"
	}

corba_tc_create_exception :: !String !String !StructMemberSeqPtr !*env -> (!TypeCodePtr, !*env)
corba_tc_create_exception repoid name members env = code {
	ccall corba_tc_exception "SSI:VI:A"
	}

corba_tc_create_recursive :: !String !*env -> (!TypeCodePtr, !*env)
corba_tc_create_recursive repoid env = code {
	ccall corba_tc_recursive "S:VI:A"
	}

//
// Accessor functions
//

corba_tc_get_kind :: !TypeCodePtr !*env -> (!Int, !*env)
corba_tc_get_kind tc env = code {
	ccall corba_tc_get_kind "I:VI:A"
	}

corba_tc_get_id :: !TypeCodePtr !*env -> (!String, !*env)
corba_tc_get_id tc env = code {
	ccall corba_tc_get_id "I:VS:A"
	}

corba_tc_get_name :: !TypeCodePtr !*env -> (!String, !*env)
corba_tc_get_name tc env = code {
	ccall corba_tc_get_name "I:VS:A"
	}

corba_tc_get_member_count :: !TypeCodePtr !*env -> (!Int, !*env)
corba_tc_get_member_count tc env = code {
	ccall corba_tc_get_member_count "I:VI:A"
	}

corba_tc_get_member_name :: !TypeCodePtr !Int !*env -> (!String, !*env)
corba_tc_get_member_name tc index env = code {
	ccall corba_tc_get_member_name "II:VS:A"
	}

corba_tc_get_member_type :: !TypeCodePtr !Int !*env -> (!TypeCodePtr, !*env)
corba_tc_get_member_type tc index env = code {
	ccall corba_tc_get_member_type "II:VI:A"
	}

corba_tc_get_member_label :: !TypeCodePtr !Int !*env -> (!AnyPtr, !*env)
corba_tc_get_member_label tc index env = code {
	ccall corba_tc_get_member_label "II:VI:A"
	}

corba_tc_get_discriminator_type :: !TypeCodePtr !*env -> (!TypeCodePtr, !*env)
corba_tc_get_discriminator_type tc env = code {
	ccall corba_tc_get_discriminator_type "I:VI:A"
	}

corba_tc_get_default_index :: !TypeCodePtr !*env -> (!Int, !*env)
corba_tc_get_default_index tc env = code {
	ccall corba_tc_get_discriminator_type "I:VI:A"
	}

corba_tc_get_length :: !TypeCodePtr !*env -> (!Int, !*env)
corba_tc_get_length tc env = code {
	ccall corba_tc_get_length "I:VI:A"
	}

corba_tc_get_content_type :: !TypeCodePtr !*env -> (!TypeCodePtr, !*env)
corba_tc_get_content_type tc env = code {
	ccall corba_tc_get_content_type "I:VI:A"
	}

////////////////////////////////
//          CORBA::Any
////////////////////////////////

:: AnyPtr :== Int

//
// Creation functions
//

corba_any_create_null :: !*env -> (!AnyPtr, !*env)
corba_any_create_null env = code {
	ccall corba_any_create_null ":VI:A"
	}

corba_any_create_short :: !Int !*env -> (!AnyPtr, !*env)
corba_any_create_short value env = code {
	ccall corba_any_create_short "I:VI:A"
	}

corba_any_create_ushort :: !Int !*env -> (!AnyPtr, !*env)
corba_any_create_ushort value env = code {
	ccall corba_any_create_ushort "I:VI:A"
	}

corba_any_create_long :: !Int !*env -> (!AnyPtr, !*env)
corba_any_create_long value env = code {
	ccall corba_any_create_long "I:VI:A"
	}

corba_any_create_ulong :: !Int !*env -> (!AnyPtr, !*env)
corba_any_create_ulong value env = code {
	ccall corba_any_create_ulong "I:VI:A"
	}

corba_any_create_float :: !Real !*env -> (!AnyPtr, !*env)
corba_any_create_float value env = code {
	ccall corba_any_create_float "R:VI:A"
	}

corba_any_create_double :: !Real !*env -> (!AnyPtr, !*env)
corba_any_create_double value env = code {
	ccall corba_any_create_double "R:VI:A"
	}

corba_any_create_bool :: !Bool !*env -> (!AnyPtr, !*env)
corba_any_create_bool value env = code {
	ccall corba_any_create_bool "I:VI:A"
	}

corba_any_create_char :: !Char !*env -> (!AnyPtr, !*env)
corba_any_create_char value env = code {
	ccall corba_any_create_char "I:VI:A"
	}

corba_any_create_octet :: !Int !*env -> (!AnyPtr, !*env)
corba_any_create_octet value env = code {
	ccall corba_any_create_octet "I:VI:A"
	}

corba_any_create_string :: !String !*env -> (!AnyPtr, !*env)
corba_any_create_string value env = code {
	ccall corba_any_create_string "S:VI:A"
	}

corba_any_create_any :: !AnyPtr !*env -> (!AnyPtr, !*env)
corba_any_create_any value env = code {
	ccall corba_any_create_any "I:VI:A"
	}

corba_any_create_tc :: !TypeCodePtr !*env -> (!AnyPtr, !*env)
corba_any_create_tc value env = code {
	ccall corba_any_create_tc "I:VI:A"
	}

corba_any_create_objref :: !CorbaObjectPtr !*env -> (!AnyPtr, !*env)
corba_any_create_objref value env = code {
	ccall corba_any_create_objref "I:VI:A"
	}

corba_any_delete :: !AnyPtr !*env -> !*env
corba_any_delete any env = code {
	ccall corba_any_delete "I:V:A"
	}

//
// Extraction functions
//

corba_any_extract_short :: !AnyPtr !*env -> (!Int, !*env)
corba_any_extract_short any env = code {
	ccall corba_any_extract_short "I:VI:A"
	}

corba_any_extract_long :: !AnyPtr !*env -> (!Int, !*env)
corba_any_extract_long any env = code {
	ccall corba_any_extract_long "I:VI:A"
	}

corba_any_extract_ushort :: !AnyPtr !*env -> (!Int, !*env)
corba_any_extract_ushort any env = code {
	ccall corba_any_extract_ushort "I:VI:A"
	}

corba_any_extract_ulong :: !AnyPtr !*env -> (!Int, !*env)
corba_any_extract_ulong any env = code {
	ccall corba_any_extract_ulong "I:VI:A"
	}

corba_any_extract_float :: !AnyPtr !*env -> (!Real, !*env)
corba_any_extract_float any env = code {
	ccall corba_any_extract_float "I:VR:A"
	}

corba_any_extract_double :: !AnyPtr !*env -> (!Real, !*env)
corba_any_extract_double any env = code {
	ccall corba_any_extract_double "I:VR:A"
	}

corba_any_extract_boolean :: !AnyPtr !*env -> (!Bool, !*env)
corba_any_extract_boolean any env = code {
	ccall corba_any_extract_boolean "I:VI:A"
	}

corba_any_extract_char :: !AnyPtr !*env -> (!Char, !*env)
corba_any_extract_char any env = code {
	ccall corba_any_extract_char "I:VI:A"
	}

corba_any_extract_octet :: !AnyPtr !*env -> (!Int, !*env)
corba_any_extract_octet any env = code {
	ccall corba_any_extract_octet "I:VI:A"
	}

corba_any_extract_string :: !AnyPtr !*env -> (!String, !*env)
corba_any_extract_string any env = code {
	ccall corba_any_extract_string "I:VS:A"
	}

corba_any_extract_any :: !AnyPtr !*env -> (!AnyPtr, !*env)
corba_any_extract_any any env = code {
	ccall corba_any_extract_any "I:VI:A"
	}

corba_any_extract_tc :: !AnyPtr !*env -> (!TypeCodePtr, !*env)
corba_any_extract_tc any env = code {
	ccall corba_any_extract_tc "I:VI:A"
	}

corba_any_extract_objref :: !AnyPtr !*env -> (!CorbaObjectPtr, !*env)
corba_any_extract_objref any env = code {
	ccall corba_any_extract_objref "I:VI:A"
	}

// Obtain the TypeCode of the Any
corba_any_get_type :: !AnyPtr !*env -> (!TypeCodePtr, !*env)
corba_any_get_type any env = code {
	ccall corba_any_get_type "I:VI:A"
	}

// DynStruct interface

:: NameValuePairSeq :== Int

corba_nv_pair_seq_create :: !Int !*env -> (!NameValuePairSeq, !*env)
corba_nv_pair_seq_create size env = code {
	ccall corba_nv_pair_seq_create "I:VI:A"
	}

// Fill a member of the sequence
corba_nv_pair_seq_fill :: !NameValuePairSeq !Int !String !AnyPtr !*env -> !*env
corba_nv_pair_seq_fill seq index name value env = code {
	ccall corba_nv_pair_seq_fill "IISI:V:A"
	}

corba_nv_pair_seq_length :: !NameValuePairSeq !*env -> (!Int, !*env)
corba_nv_pair_seq_length seq env = code {
	ccall corba_nv_pair_seq_length "I:VI:A"
	}

corba_nv_pair_seq_get :: !NameValuePairSeq !Int !*env -> (!String, !AnyPtr, !*env)
corba_nv_pair_seq_get seq index env = code {
	ccall corba_nv_pair_seq_get "II:VSI:A"
	}

corba_nv_pair_seq_delete :: !NameValuePairSeq !*env -> !*env
corba_nv_pair_seq_delete seq env = code {
	ccall corba_nv_pair_seq_delete "I:V:A"
	}

// Answers the Any extracted from the DynStruct
corba_dyn_struct_create :: !TypeCodePtr !NameValuePairSeq !*env -> (!AnyPtr, !*env)
corba_dyn_struct_create tc members env = code {
	ccall corba_dyn_struct_create "II:VI:A"
	}

// Answer the member list extracted from the DnyStruct
corba_dyn_struct_members :: !AnyPtr !*env -> (!NameValuePairSeq, !*env)
corba_dyn_struct_members any env = code {
	ccall corba_dyn_struct_members "I:VI:A"
	}

// DynSequence Interface

:: AnySeq :== Int

corba_any_seq_create :: !Int !*env -> (!AnySeq, !*env)
corba_any_seq_create size env = code {
	ccall corba_any_seq_create "I:VI:A"
	}

// Fill a member of the sequence
corba_any_seq_fill :: !AnySeq !Int !AnyPtr !*env -> !*env
corba_any_seq_fill seq index value env = code {
	ccall corba_any_seq_fill "III:V:A"
	}

corba_any_seq_length :: !AnySeq !*env -> (!Int, !*env)
corba_any_seq_length seq env = code {
	ccall corba_any_seq_length "I:VI:A"
	}

corba_any_seq_get :: !AnySeq !Int !*env -> (!AnyPtr, !*env)
corba_any_seq_get seq index env = code {
	ccall corba_any_seq_get "II:VI:A"
	}

corba_any_seq_delete :: !AnySeq !*env -> !*env
corba_any_seq_delete seq env = code {
	ccall corba_any_seq_delete "I:V:A"
	}

// Answers the Any extracted from the DynSequence
corba_dyn_sequence_create :: !TypeCodePtr !AnySeq !*env -> (!AnyPtr, !*env)
corba_dyn_sequence_create tc elements env = code {
	ccall corba_dyn_sequence_create "II:VI:A"
	}

// Answer the contents of the DnyStruct
corba_dyn_sequence_elements :: !AnyPtr !*env -> (!AnySeq, !*env)
corba_dyn_sequence_elements any env = code {
	ccall corba_dyn_sequence_elements "I:VI:A"
	}

// DynUnion Interface

// Answers the Any extracted from the DynUnion
corba_dyn_union_create :: !TypeCodePtr !AnyPtr !AnyPtr !*env -> (!AnyPtr, !*env)
corba_dyn_union_create tc label value env = code {
	ccall corba_dyn_union_create "III:VI:A"
	}

// Answer the discriminator and the active member of the DynUnion
// If there is no active member, and Any with typecode tk_null is returned
corba_dyn_union_get :: !AnyPtr !*env -> (!AnyPtr, !AnyPtr, !*env)
corba_dyn_union_get any env = code {
	ccall corba_dyn_union_get "I:VII:A"
	}

//
// DynEnum Interface
//

// Answers the Any extracted from the DynEnum
corba_dyn_enum_create :: !TypeCodePtr !String !*env -> (!AnyPtr, !*env)
corba_dyn_enum_create tc value env = code {
	ccall corba_dyn_enum_create "IS:VI:A"
	}

// Answer the value of the DynEnum
corba_dyn_enum_get :: !AnyPtr !*env -> (!String, !*env)
corba_dyn_enum_get any env = code {
	ccall corba_dyn_enum_get "I:VS:A"
	}

//
// DynArray Interface
//

// Answers the Any extracted from the DynArray
corba_dyn_array_create :: !TypeCodePtr !AnySeq !*env -> (!AnyPtr, !*env)
corba_dyn_array_create tc elements env = code {
	ccall corba_dyn_array_create "II:VI:A"
	}

// Answer the contents of the DnyArray
corba_dyn_array_elements :: !AnyPtr !*env -> (!AnySeq, !*env)
corba_dyn_array_elements any env = code {
	ccall corba_dyn_array_elements "I:VI:A"
	}

////////////////////////////////
//      EXCEPTION
////////////////////////////////

:: ExceptionPtr :== Int

// These should be equal to the CompletionStatus values used by the ORB 
// implementation
COMPLETED_YES 	:== 0
COMPLETED_NO 	:== 1
COMPLETED_MAYBE	:== 2

corba_exception_is_system :: !ExceptionPtr !*env -> (!Bool, !*env)
corba_exception_is_system ex env = code {
	ccall corba_exception_is_system "I:VI:A"
	}

// Only for system exceptions
corba_exception_get_minor :: !ExceptionPtr !*env -> (!Int, !*env)
corba_exception_get_minor ex env = code {
	ccall corba_exception_get_minor "I:VI:A"
	}

// Only for system exceptions
corba_exception_get_completed :: !ExceptionPtr !*env -> (!Int, !*env)
corba_exception_get_completed ex env = code {
	ccall corba_exception_get_completed "I:VI:A"
	}

corba_exception_get_repoid :: !ExceptionPtr !*env -> (!String, !*env)
corba_exception_get_repoid ex env = code {
	ccall corba_exception_get_repoid "I:VS:A"
	}

:: UnknownExPtr :== Int

// Narrow the exception to UnknownUserException, or return 0
corba_unknown_ex_narrow :: !ExceptionPtr !*env -> (!UnknownExPtr, !*env)
corba_unknown_ex_narrow ex env = code {
	ccall corba_unknown_ex_narrow "I:VI:A"
	}

// Answer the repoid of the user exception contained in the exception
corba_unknown_ex_get_repoid :: !UnknownExPtr !*env -> (!String, !*env)
corba_unknown_ex_get_repoid ex env = code {
	ccall corba_unknown_ex_get_repoid "I:VS:A"
	}

corba_unknown_ex_get_value :: !UnknownExPtr !TypeCodePtr !*env -> (!AnyPtr, !*env)
corba_unknown_ex_get_value ex tc env = code {
	ccall corba_unknown_ex_get_value "II:VI:A"
	}

////////////////////////////////
//    NamedValue and NVList
////////////////////////////////

:: NamedValuePtr :== Int

// Values for the Flags field in the NamedValue object
// These should be equal to the ArgModeFlags values used by the ORB 
// implementation

:: ArgModeFlags :== Int

ARG_IN 		:== 1
ARG_OUT 	:== 2
ARG_INOUT	:== 4

corba_nv_get_name :: !NamedValuePtr !*env -> (!String, !*env)
corba_nv_get_name nv env = code {
	ccall corba_nv_get_name "I:VS:A"
	}

corba_nv_get_value :: !NamedValuePtr !*env -> (!AnyPtr, !*env)
corba_nv_get_value nv env = code {
	ccall corba_nv_get_value "I:VI:A"
	}

corba_nv_get_flags :: !NamedValuePtr !*env -> (!ArgModeFlags, !*env)
corba_nv_get_flags nv env = code {
	ccall corba_nv_get_flags "I:VI:A"
	}

corba_nv_set_type :: !NamedValuePtr !TypeCodePtr !*env -> !*env
corba_nv_set_type nv tc env = code {
	ccall corba_nv_set_type "II:V:A"
	}

corba_nv_set_value :: !NamedValuePtr !AnyPtr !*env -> !*env
corba_nv_set_value nv value env = code {
	ccall corba_nv_set_value "II:V:A"
	}

:: NVListPtr :== Int

corba_nvlist_create :: !*env -> (!NVListPtr, !*env)
corba_nvlist_create env = code {
	ccall corba_nvlist_create ":VI:A"
	}

corba_nvlist_add :: !NVListPtr !ArgModeFlags !*env -> (!NamedValuePtr, !*env)
corba_nvlist_add nvlist flags env = code {
	ccall corba_nvlist_add "II:VI:A"
	}	

corba_nvlist_count :: !NVListPtr !*env -> (!Int, !*env)
corba_nvlist_count nvlist env = code {
	ccall corba_nvlist_count "I:VI:A"
	}

corba_nvlist_get :: !NVListPtr !Int !*env -> (!NamedValuePtr, !*env)
corba_nvlist_get nvlist index env = code {
	ccall corba_nvlist_get "II:VI:A"
	}

//
// Interface for CORBA::Request
//

:: RequestPtr :== Int

corba_object_create_request :: !CorbaObjectPtr !String 
							!*env -> (!RequestPtr, !*env)
corba_object_create_request obj operation env = code {
	ccall corba_object_create_request "IS:VI:A"
	}

corba_request_arguments :: !RequestPtr !*env -> (!NVListPtr, !*env)
corba_request_arguments req env = code {
	ccall corba_request_arguments "I:VI:A"
	}

corba_request_add_in_arg :: !RequestPtr !String !AnyPtr !*env -> !*env
corba_request_add_in_arg req name value env = code {
	ccall corba_request_add_in_arg "ISI:V:A"
	}

corba_request_add_out_arg :: !RequestPtr !String !AnyPtr !*env -> !*env
corba_request_add_out_arg req name value env = code {
	ccall corba_request_add_out_arg "ISI:V:A"
	}

corba_request_add_inout_arg :: !RequestPtr !String !AnyPtr !*env -> !*env
corba_request_add_inout_arg req name value env = code {
	ccall corba_request_add_inout_arg "ISI:V:A"
	}

corba_request_add_out_arg_template :: !RequestPtr !String !TypeCodePtr !*env -> !*env
corba_request_add_out_arg_template req name tc env = code {
	ccall corba_request_add_out_arg_template "ISI:V:A"
	}

corba_request_invoke :: !RequestPtr !*env -> !*env
corba_request_invoke req env = code {
	ccall corba_request_invoke "I:V:A"
	}

// Set the type of expected result
corba_request_set_result_tc :: !RequestPtr !TypeCodePtr !*env -> !*env
corba_request_set_result_tc req tc env = code {
	ccall corba_request_set_result_tc "II:V:A"
	}

corba_request_result :: !RequestPtr !*env -> (!AnyPtr, !*env)
corba_request_result req env = code {
	ccall corba_request_result "I:VI:A"
	}

corba_request_exception :: !RequestPtr !*env -> (!ExceptionPtr, !*env)
corba_request_exception req env = code {
	ccall corba_request_exception "I:VI:A"
	}

corba_request_add_exception_def :: !RequestPtr !TypeCodePtr !*env -> !*env
corba_request_add_exception_def req tc env = code {
	ccall corba_request_add_exception_def "II:V:A"
	}

corba_request_delete :: !RequestPtr !*env -> !*env
corba_request_delete req env = code {
	ccall corba_request_delete "I:V:A"
	}

////////////////////////////////
//     SERVER REQUEST
////////////////////////////////

:: ServerRequestPtr :== Int

corba_server_request_operation :: !ServerRequestPtr !*env -> (!String, !*env)
corba_server_request_operation req env = code {
	ccall corba_server_request_operation "I:VS:A"
	}

corba_server_request_set_params :: !ServerRequestPtr !NVListPtr !*env -> !*env
corba_server_request_set_params req args env = code {
	ccall corba_server_request_set_params "II:V:A"
	}

corba_server_request_set_result :: !ServerRequestPtr !AnyPtr !*env -> !*env
corba_server_request_set_result req result env = code {
	ccall corba_server_request_set_result "II:V:A"
	}

corba_server_request_set_exception :: !ServerRequestPtr !AnyPtr !*env -> !*env
corba_server_request_set_exception req result env = code {
	ccall corba_server_request_set_exception "II:V:A"
	}

corba_server_request_get_args :: !ServerRequestPtr !*env -> (!NVListPtr, !*env)
corba_server_request_get_args req env = code {
	ccall corba_server_request_get_args "I:VI:A"
	}

////////////////////////////////
//      SERVER SIDE
////////////////////////////////

corba_listener_init :: !*env -> !*env
corba_listener_init env = code {
	ccall corba_listener_init ":V:A"
	}

corba_listener_receive :: !*env -> (!ServerRequestPtr, !CorbaObjectPtr, !*env)
corba_listener_receive env = code {
	ccall corba_listener_receive ":VII:A"
	}

corba_listener_reply :: !*env -> !*env
corba_listener_reply env = code {
	ccall corba_listener_reply ":V:A"
	}

corba_servant_create :: !String !*env -> (!CorbaObjectPtr, !*env)
corba_servant_create repoid env = code {
	ccall corba_servant_create "S:VI:A"
	}

// Local Variables:
// tab-width: 4
// End:

