
/*
 * This file implements a wrapper program around idl2clean. It implements
 * things which can't be done from Clean, like starting the IR daemon
 * and running the MICO idl compiler.
 */

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>

#ifdef _WINDOWS
#include <process.h>
#include <windows.h>
#endif

const char * iraddr = "inet:localhost:4141";

int
main(int argc, char * argv[])
{
  char input_file[1024];

  if (argc != 2) {
	printf("Usage: idl2clean <IDL FILE>\n");
	exit(1);
  }

  strcpy(input_file, argv[1]);

  // Construct a filename for the Clean file
  // Eg. /foo/bar/x.idl -> x
  char module_name[1024];
  char tmp[1024];
  strcpy(tmp, input_file);

  // Check extension
  if ((strlen(tmp) < 4) || 
	  (strcmp(&tmp[strlen(tmp) -4], ".idl") != 0)) {
    printf("Error: Input file must have extension .idl.\n");
	exit(1);
  }

  // Remove extension
  tmp[strlen(tmp) - 4] = '\0';

  // Remove directories
  int i;
  i = strlen(tmp) - 1;
  while ((i > 0) && (tmp[i] != '/') && (tmp[i] != '\\'))
	i--;
  if (i > 0)
	i++;

  strcpy(module_name, &(tmp[i]));
  
  printf("Module name: %s\n", module_name); fflush(stdout);

#ifdef _WINDOWS
  system("pv.exe -fk ird.exe");
#else
  system("killall -q ird");

  sleep(1);
#endif

  printf("Starting IR daemon...\n");

#ifdef _WINDOWS
  int err = _spawnlp(_P_NOWAITO, "ird.exe", "ird.exe", "-ORBIIOPAddr", iraddr, NULL);
  if (err == -1) {
	printf("Error: Unable to start ird: %s\n", strerror(errno));
	exit(1);
  }
#else
  int err = system("ird -ORBIIOPAddr inet:localhost:4141 &");

  sleep(1);
#endif // _WINDOWS

  printf("Running the MICO IDL compiler...\n"); fflush(stdout);

#ifdef _WINDOWS
  err = _spawnlp(_P_WAIT, "idl.exe", "-I.", "--no-codegen-c++", "--feed-included-defs", 
				 "--feed-ir", "-ORBIfaceRepoAddr", iraddr, input_file, NULL);
  if (err == -1) {
	printf("Error: Unable to start idl compiler: %s\n", strerror(errno));
	exit(1);
  }
#else
  char command[1024];
  sprintf(command, "idl -I. --no-codegen-c++ --feed-included-defs --feed-ir -ORBIfaceRepoAddr inet:localhost:4141 %s", input_file);
  err = system(command);
#endif // _WINDOWS

  if (err != 0) {
	printf("Error: Unable to feed IDL file to the Interface Repository.\n");
	exit(1);
  }

  printf("Running the IDL-CLEAN compiler...\n"); fflush(stdout);

#ifdef _WINDOWS
  _spawnlp(_P_WAIT, "idl2clean_main", "-nt", "-nr", iraddr, module_name, NULL);
#else
  sprintf(command, "idl2clean_main -nt -nr inet:localhost:4141 %s", module_name);
  system(command);
#endif

  return 0;
}



// Local Variables:
// tab-width: 4
// End:



